close all;
% Constants
fs = 192E3;
fc = 38.4E3;
fm = 6.4E3;

L_code = 511;

preamble_bpsk = 2*getGold(100,1,L_code) - 1;
code_bpsk = 2*getGold(1,1,L_code) - 1;


P = fs/fm;
t = (0:P*2*L_code-1)/fs;
c = cos(2*pi*fc.*t);

Ls = L_code*P*2;
diff = 0;

detector = Detector(fs,fc,fm,preamble_bpsk,code_bpsk);
detector2 = Detector2(fs,fc,fm,preamble_bpsk,code_bpsk);


N_fir = 100;
fp = fm/6;
rp = 0.001;
rst = 1E-6;
fir = firDesign(N_fir, fp, fs, rp, rst);

demodulator = Demodulator_old(fs,fc,fm,L_code*P,L_code*P*4,0,0,fir);
correlator = Correlator_old(fs,fc,fm,L_code,0,0,getGold(1,1,L_code));

%%
tic
%all_gold_bpsk = 2*getGold(2,120,L_code)-1;
toc

%%
num_noise_level = 25;
num_tests = 500;

noise_level = [1 2 3 4 5 6 7 8 9 10 12 14 16 18 20 25 30 35 40 45 55 65 75 90 110];

hits = zeros(num_tests,num_noise_level);
hits2 = zeros(num_tests,num_noise_level);
hits3 = zeros(num_tests,num_noise_level);
snr_tot = zeros(1,num_noise_level);

tmp_snr = zeros(1,num_tests);


fprintf('\nCurrent noise_level: 1');
for i = 1:num_noise_level
    for j = 1:num_tests
        noise_bpsk = all_gold_bpsk(1:noise_level(i),:);
        totnoise_bpsk = sum([noise_bpsk(:,1:j) noise_bpsk noise_bpsk(:,j:end-1)],1);
        totcode_bpsk = [preamble_bpsk code_bpsk(1,:)];
        
        sumcode_bpsk = totcode_bpsk + totnoise_bpsk;
        
        tmp_snr(j) = snr(totcode_bpsk,totnoise_bpsk);
        
        phi = upsample(sumcode_bpsk, P);
        for k = 1:P:length(phi)
            phi(k:k+P-1) = phi(k);
        end

        s = phi.*c;
        
        diff = j;
        s_buffer=[zeros(1,diff) s zeros(1,Ls-diff)];

        detection = detector.step(s_buffer);
        detection2 = detector2.step(s_buffer);
        
        [s_demod, offset] = demodulator.step(s_buffer);
        detection3 = correlator.step(s_demod);
        detection3(1,2) = detection(1,2) + offset - 35;
        
        if detection(1,1) > 0 && abs(diff - detection(1,2)) < 10
            hits(j,i) = 1;
        end
        
        if detection2(1,1) > 0 && abs(diff - detection2(1,2)) < 10
            hits2(j,i) = 1;
        end
        
        if detection3(1,1) > 0 && abs(diff - detection3(1,2)) < 10
            hits3(j,i) = 1;
        end
    end
    snr_tot(i) = 10*log10(mean(10.^(tmp_snr./10)));
    fprintf('\b\b%2i',i+1);
end
fprintf('\n');
%%
p_err = 1 - mean(hits,1);
p_err2 = 1 - mean(hits2,1);
p_err3 = 1 - mean(hits3,1);

h1 = figure;
semilogy(snr_tot(end:-1:1),p_err(end:-1:1));
hold on;
semilogy(snr_tot(end:-1:1),p_err2(end:-1:1));
semilogy(snr_tot(end:-1:1),p_err3(end:-1:1));
title('Simulated BER for three correlator implementations');
xlabel('SNR [dB]');
ylabel('Probability of bit error');
legend({'All decimated' 'Full rate sync' 'Costas-based'});
text(-19, 1.35*10^-3,'\fontsize{15}{0} \selectfont $L_{code} = 511 ~~~~~~~ N_{iterations} = 500 ~~~~~~ $Tolerance: $\pm 20$ samples');
grid on;
fig1 = Plotfix(h1);
%%
%fig1.print('impl-process-simulated-ber_5',1);
