classdef Correlator < handle
    %CORRELATOR Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        
        % Input variables
        fs;
        fs_oversample;
        fc;
        fm;
        baseband;
        code;
        
        % Internals
        l_message;
        n_code;
        l_code;
        l_xc;
        l_code_upsampled;
        
        code_upsampled;
        
        modulated_threshold;
        baseband_threshold;
        
        % Internal constants
        modulated_threshold_factor = 11; %12
        baseband_threshold_factor = 25;
        
        plot_crosscorr = false;
       
        
    end
    
    methods
        function self = Correlator(fs, fc, fm, l_message, baseband, fs_oversample, code)
            % Assign inputs
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.l_message = l_message;
            self.baseband = baseband;
            self.fs_oversample = fs_oversample;
            self.code = code;
            
            % Initialize
            [self.n_code, self.l_code] = size(code);
            
            self.modulated_threshold = self.l_code/self.modulated_threshold_factor;
            self.baseband_threshold = self.l_code/self.baseband_threshold_factor;
            
            if baseband
                code_modulated = 2.*code - 1;
                P = self.fs_oversample/self.fm;
                self.l_code_upsampled = self.l_code*P;
                self.code_upsampled = zeros(self.n_code,self.l_code*P);
                for code_number = 1:self.n_code
                    for code_index = 1:self.l_code-1
                        self.code_upsampled(code_number,ceil(P*code_index-(P-1))) = code_modulated(code_number,code_index);
                        for j = 1:P-1
                            self.code_upsampled(code_number,ceil(P*code_index-(P-1)+j)) = code_modulated(code_number,code_index);
                        end
                    end
                end
            else
                %self.l_xc = self.l_code + self.l_message/(self.fs/self.fm) - 1;
                self.l_xc = self.l_code + 2*self.l_code - 1;
            end
            
            
        end
        
        function detection = step(self, s_demod)
            detection = zeros(self.n_code,3) - 1;
            
            if self.baseband
                for current_code = 1:self.n_code
                    xc = xcorr(self.code_upsampled(current_code,:),s_demod);
                    [level, index] = max(xc);

                    if  level > self.baseband_threshold
                        detection(current_code,1) = true;
                        detection(current_code,2) = round((self.l_code_upsampled-index)*self.fs/self.fs_oversample);
                        
                    end
                end
                
            else % Non-baseband
                for current_code = 1:self.n_code
                    xc = xcorr(self.code(current_code,:),s_demod);
                    
                    for i = 2:self.l_xc
                        if abs(xc(i-1) - xc(i)) > self.modulated_threshold
                            detection(current_code,1) = true;   % Detected = true
                            %detection(current_code,2) = (self.l_code-i)*self.fs/self.fm; % Lag
                            detection(current_code,2) = (2*self.l_code-i)*self.fs/self.fm; % Lag
                            detection(current_code,3) = abs(xc(i-1) - xc(i)); % Threshold
                            
                            break;
                        end
                    end
                    
                    if self.plot_crosscorr;
                        figure(3);
                        plot(abs(xc))
                        %hold all;
                        ylim([-40 250]);
                        if detection(current_code,1) == true;
                            set(gca,'Color',[1 .9 .9]);
                        end
                        title('Demodulated signal/code correlation');
                        
                    end
                end
                    
            end
            
        end
        
        function plotCrosscorr(self,enable)
            if enable > 0
                self.plot_crosscorr = true;
            else
                self.plot_crosscorr = false;
            end
        end
    end
end

