classdef Correlator_complex < handle
    %CORRELATOR Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        
        % Input variables
        fs;
        fs_oversample;
        fc;
        fm;
        baseband;
        code;
        
        % Internals
        l_buffer;
        n_code;
        l_code;
        l_xc;
        l_code_upsampled;
        
        code_upsampled;
        P;
        N_fine;
        N_fine_factor = 100;
        I_start_fine;
        
        modulated_threshold;
        baseband_threshold;
        
        % Internal constants
        modulated_threshold_factor =200; %12
        baseband_threshold_factor = 25;
        
        plot_crosscorr = false;
       
        
    end
    
    methods
        function self = Correlator_complex(fs, fc, fm, l_buffer, code)
            % Assign inputs
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.l_buffer = l_buffer;
            self.code = code;
   
            
            % Initialize
            [self.n_code, self.l_code] = size(code); 
            
            self.modulated_threshold = self.l_code/self.modulated_threshold_factor;
            
            self.l_xc = self.l_code*4-1; % The buffer is to codes long
            
            % Upsampled code for fine tracking
            self.P = self.fs/self.fm; % Upsampling factor
            self.N_fine = self.N_fine_factor*self.P; % Length of fine crosscorr
            self.I_start_fine = self.l_code*self.P/2; % Midpoint of code
            
            self.code_upsampled = upsample(code,self.P);
            for i = 1:self.P:self.l_code*self.P
                self.code_upsampled(i:i+self.P-1) = self.code_upsampled(i);
            end
            
            
        end
        
        function detection = step(self, m, r_lp)
            detection = zeros(self.n_code,3) - 1;
            for current_code = 1:self.n_code
                xc = abs(xcorr(self.code(current_code,:),m));
                [~, i] = max(xc);
                if abs(xc(i-1) - xc(i)) > 2 %self.modulated_threshold


                    coarse_lag = (self.l_xc/2-i)*self.fs/self.fm; % Lag
                    fprintf('Coarse_lag: %i, ', coarse_lag);
                    i_fine_start = coarse_lag;
                    %fprintf('i_fine_start: %i\n', i_fine_start);
                    if i_fine_start > 0 && i_fine_start < self.l_buffer - self.N_fine


                        xc_fine = abs(xcorr(self.code_upsampled(1:self.N_fine),r_lp(i_fine_start:i_fine_start+self.N_fine)));
                        [~, i_max_fine] = max(xc_fine);

                        fine_offset = self.N_fine - i_max_fine;
                        fine_lag =  coarse_lag + fine_offset;
                        detection(current_code,1) = true;   % Detected = true
                        detection(current_code,2) = fine_lag;
                        detection(current_code,3) = abs(xc(i-1) - xc(i)); % Threshold
                        fprintf('Fine_offset %i, Fine_lag %i\n', fine_offset, fine_lag);
                        
                        if 0
                            figure;
                            plot(xc);
                            title('correlator xc');

                            figure;
                            plot(abs(xc_fine));
                            title('correlator xc fine');

                            figure;
                            plot(abs(r_lp));
                            title('correlator r lp');
                            %figure;
                            %plot(abs(r_lp(i_fine_start:i_fine_start+self.N_fine)));
                            %hold on
                            %plot(abs(self.code_upsampled(1:self.N_fine)));
                            %figure;
                            %plot(abs(xc_fine));
                        end
                    end

                end
                
                
                if self.plot_crosscorr;
                    figure(3);
                    plot(xc)
                    %hold all;
                    %ylim([-40 250]);
                    if detection(current_code,1) == true;
                        set(gca,'Color',[1 .9 .9]);
                    end
                    title('Demodulated signal/code correlation');
                end
            end

            
        end
        
        function plotCrosscorr(self,enable)
            if enable > 0
                self.plot_crosscorr = true;
            else
                self.plot_crosscorr = false;
            end
        end
    end
end

