classdef Demodulator < handle
    %BPSKDEMODULATOR Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        
        
        % Input variables
        fs;
        fc;
        fm;
        fir;
        l_signal;
        l_buffer;
        baseband;
        fs_oversample;
        
        % Internals
        plot_detectionPoints = false;
        carrier_unsynced;
        
        % Internal constants
        l_corr;
        
        % Baseband 2.0
        carrier_complex;
        fir_baseband;
        
        % Non-baseband demod
        xcorr_start_idx;
        n_xcorr_idx;
        l_xcorr;
        start_offset;
        
        % 3rd gen demod
        NFFT = 64;
        src_step_factor = 1/10;
        
        src_step;
        src_samples;
        l_src;
        
        fft_c_idx;
        
        costas_t;
        costas_c_T;
        costas_fs_T;
        costas_phi_avg_start;
        costas_phi_avg_stop;
        costas_c_t;
        
        l_costas;
        c;
        
        Hd;
        
    end
    
    methods
        function self = Demodulator(fs, fc, fm, l_signal,l_buffer, baseband, fs_oversample, fir)
            % Assign inputs
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.l_signal = l_signal;
            self.l_buffer = l_buffer;
            self.baseband = baseband;
            self.fs_oversample = fs_oversample;
            self.fir = fir;
            
            
            self.l_corr = round(self.l_signal/2);
            
            % Initialize
            %t = 0:1/self.fs:(l_buffer+self.l_corr)/self.fs; % Length of m + crosscorr
            t = (0:1:(l_buffer-1))/self.fs;
            self.carrier_unsynced = cos(2*pi*self.fc*t);         % Carrier\
            self.c = cos(2*pi*self.fc*t);
            % Baseband 2.0
            t2 = 0:1/self.fs:l_buffer/self.fs-1/self.fs;
            
            
            self.fir_baseband = firDesign(100,2.5*self.fm,self.fs,0.001,1E-6);
            
            % Non-baseband demod
            self.l_xcorr = round(self.fc/self.fm);
            interval = round(self.l_buffer/25);
            self.xcorr_start_idx = 1:self.l_xcorr+1:(self.l_xcorr+1)*self.l_xcorr;
            for i = 1:24
                self.xcorr_start_idx = [self.xcorr_start_idx, interval*i:self.l_xcorr+1:(self.l_xcorr+1)*self.l_xcorr + interval*i];
            end
            self.n_xcorr_idx = length(self.xcorr_start_idx);
            %self.start_offset = round(self.l_buffer/2 - self.n_xcorr_idx*self.l_xcorr);
            self.start_offset = 1;
            
            
            
            % 3rd gen demod costas loop
            self.src_step = round(self.l_signal*self.src_step_factor);      % Steps for sample vector
            self.src_samples = 1:self.src_step:self.l_signal-1;             % Start sample vector
            self.l_src = length(self.src_samples);
            self.fft_c_idx = round(self.fc/(self.fs/self.NFFT)+1);
            
            self.costas_t = 0:1:self.l_signal-1;
            self.costas_c_T = 1/self.fc;
            self.costas_fs_T = 1/self.fs;
            self.costas_phi_avg_start = round(self.l_signal/4);
            self.costas_phi_avg_stop = round(self.costas_phi_avg_start + self.l_signal/4);
            
            self.costas_c_t = 0:1:self.l_buffer-1;
            
            self.l_costas = round(self.l_signal/2);
        end
        
        function [s_demod offset] = step(self, s)
            if self.baseband
                if 0
                    i = 1;
                    for n = 1:self.fs/self.fs_oversample:self.l_buffer
                        s_demod(i) = s(floor(n));
                        i = i+1;
                    end
                    
                else
                    
                    s_downmix = self.carrier_complex.*s;
                    s_filtered = step(self.fir_baseband,s_downmix');
                    s_abs = abs(s_filtered);
                    %s_bandpass = step(self.Hd,s_abs);
%                     
%                      
%                    figure;
%                     plot(real(s_filtered));
%                     
%                     figure;
%                     plot(imag(s_filtered));
%                     
%                     figure;
%                     plot(s_abs);
%                     
% %                     figure
% %                     plot(s_bandpass);
% %                     
%                     S = fft(s_abs);
%                     P2 = abs(S/self.l_buffer);
%                     P1 = P2(1:(self.l_buffer/2)+1);
%                     f = self.fs*(0:(self.l_buffer/2))/self.l_buffer;
% 
%                     figure;
%                     semilogx(f,P1);
%                     title('Single-Sided Amplitude Spectrum of received S(t)')
%                     xlabel('f (Hz)')
%                     ylabel('|P1(f)|')
%                     
%                     
%                     
%                     S = fft(imag(s_downmix));
%                     P2 = abs(S/self.l_buffer);
%                     P1 = P2(1:(self.l_buffer/2)+1);
%                     f = self.fs*(0:(self.l_buffer/2))/self.l_buffer;
% 
%                     figure;
%                     semilogx(f,P1);
%                     title('Single-Sided Amplitude Spectrum of downmix')
%                     xlabel('f (Hz)')
%                     ylabel('|P1(f)|')
                    
                    
                    
                    i = 1;
                    for n = 1:self.fs/self.fs_oversample:self.l_buffer
                        s_demod(i) = s(floor(n));
                        i = i+1;
                    end
                    offset = 0;
                end    
            else % Non-baseband
%                 % Crosscorrelate unsynced local carrier and signal OLD
%                 xc = xcorr(self.carrier_unsynced(1:self.l_corr),s(1:self.l_corr));
%                 [xcMax, xcMaxIndex] = max(xc); %Find corr. peak
%                 
%                 % Use peak index to offset carrier. Remove extra.
%                 % TODO: Some uncertainty around zero offset. Needs attention.
%                 offset = (2*self.l_corr - 1)/2 - xcMaxIndex;
%                 if offset > 0
%                     offset = ceil(offset);
%                     c = self.carrier_unsynced(self.l_corr-offset+1:end-offset-1);
% 
%                 else
%                    offset = floor(offset); 
%                    c = self.carrier_unsynced(1-offset:end-offset-1-self.l_corr);
%                 end
% __________________ 2nd gen below
                if 0
                    % Crosscorrelate unsynced local carrier and signal
                    xcorr_peak = zeros(self.n_xcorr_idx,2);
                    for i = 1:self.n_xcorr_idx
                       xc = xcorr(self.carrier_unsynced( (self.start_offset + self.xcorr_start_idx(i)) : (self.start_offset + self.xcorr_start_idx(i) + self.l_xcorr) ),s( (self.start_offset + self.xcorr_start_idx(i)) : (self.start_offset + self.xcorr_start_idx(i) + self.l_xcorr) ) );
                       xcorr_peak(i,:) = max(xc); %Find corr. peak
                    end

                    % Find the best crosscorrelation
                    [best_xcorr, best_xcorr_idx] = max(xcorr_peak(:,1));
                    if 0
                        figure(4)
                        plot(xcorr_peak(:,1));
                        ylim([0,10]);
                    end
                    % Use peak index to offset carrier. Remove extra.
                    % TODO: Some uncertainty around zero offset. Needs attention.
                    offset = (2*self.l_xcorr - 1)/2 - xcorr_peak(best_xcorr_idx,2);
                    if offset > 0
                        offset = ceil(offset);
                        c = self.carrier_unsynced(self.l_corr-offset+1:end-offset-1);

                    else
                       offset = floor(offset); 
                       c = self.carrier_unsynced(1-offset:end-offset-1-self.l_corr);
                    end
                    if 0
                        figure(3)
                        plot(s(self.l_buffer/2:self.l_buffer/2+20))
                        hold on
                        plot(c(self.l_buffer/2:self.l_buffer/2+20))
                        hold off
                    end
                end
% ___________________ 3rd gen below
                if 1
                    % Find signal start position

                    start_idx = 1;
                    for i = 1:self.l_src
                       S = abs(fft(s(self.src_samples(i):self.src_samples(i)+self.NFFT)));
                       if S(self.fft_c_idx)/mean([S(1:self.fft_c_idx-1), S(self.fft_c_idx+1:end)]) > 1.5
                           start_idx = self.src_samples(i);
                           break;
                       end
                    end


                    
                   % Run costas loop on selection
                    phi = zeros(1,self.l_costas);            % Phase vector of VCO initialize
                    s1 = zeros(1,self.l_costas); 
                    s2 = zeros(1,self.l_costas);
                    y1 = zeros(1,self.l_costas);
                    y2 = zeros(1,self.l_costas);

                    
                    for i = 1:self.l_costas

                        if i>1
                        % The step in which phase is changed is pi*5*10*-5, it can be varied.        

                            phi(i) = phi(i-1) - (100*10^-5)*pi*sign(y1(i-1)*y2(i-1));
                        end

                        s1(i) = s(start_idx+i-1) * cos(2*pi*self.fc*self.costas_t(i)/self.fs  + phi(i));
                        s2(i) = s(start_idx+i-1) * sin(2*pi*self.fc*self.costas_t(i)/self.fs  + phi(i));

                        % -----------------------INTEGRATOR------------------------------------
                        if i<=100
                        %  If sample index is less than 100 (Tc/Ts) then we sum available previous
                        %  samples
                            for j=1:i
                                y1(i) = y1(i) + s1(j);
                                y2(i) = y2(i) + s2(j);
                            end

                        else
                        % Summing previous 100 (Tc/Ts) values        
                            for j = i-99:i
                                y1(i) = y1(i) + s1(j);
                                y2(i) = y2(i) + s2(j);
                            end
                        end
                        %----------------------------------------------------------------------    
                    end 

                    costas_phase_offset = mean( phi(self.costas_phi_avg_start:self.costas_phi_avg_stop) );
                    c_time = self.costas_fs_T*(start_idx - 1);
                    c_periods = c_time/self.costas_c_T;
                    c_zero_phase = (c_periods-floor(c_periods))*2*pi;
                    c_phase = costas_phase_offset - c_zero_phase;
                    %c_check = cos(2*pi*self.fc*self.costas_c_t(self.l_costas-self.fs/self.fm:self.l_costas)/self.fs + c_phase);
                    %[~, idx] = max(c_check);
                    %decision = c_check(idx)*s(start_idx+self.l_costas-self.fs/self.fm+idx);
                    %if decision < 0
                     %   c_phase = c_phase + pi;
                    %end
                    %fprintf('Phi_costas: %1.2f, Phi_zero: %1.2f, Phi_total = %1.2f\n',costas_phase_offset, c_zero_phase, c_phase);
                    c = cos(2*pi*self.fc*self.costas_c_t/self.fs + c_phase);
                    
                if 0
                        figure(3)
                        plot(s(self.l_buffer/2:self.l_buffer/2+20))
                        hold on
                        plot(c(self.l_buffer/2:self.l_buffer/2+20))
                        hold off
                end   
                    
                   
                end
                % Multiply signal by offset car\rier.
                mul = s.*c;

                % Low-pass signal
                mulLP = step(self.fir,mul');

                % Find decision points by zero-detection
                % find max -> find zero
                if 0
                    [mulLPmax mulLPmaxIdx] = max(mulLP);
                    k = mulLPmaxIdx;
                    zero_counter = 1;
                    offset_vector = ones(1,100);
                    while k<self.l_signal-1
                        if (sign(mulLP(k)) * sign(mulLP(k+1))) == -1;
                            % Find offset by modulus with known ratio fs/fm
                            offset_vector(zero_counter) = mod(k,self.fs/self.fm)+self.fs/self.fm/2;
                            if zero_counter == 100
                                break;
                            end
                            zero_counter = zero_counter + 1;

                        end
                        k = k+1;
                    end
                end
                
                if 1
                    k = start_idx+self.src_step;
                    zero_counter = 1;
                    offset_vector = ones(1,100);
                    while k<self.l_buffer-1
                        if (sign(mulLP(k)) * sign(mulLP(k+1))) == -1;
                            % Find offset by modulus with known ratio fs/fm
                            offset_vector(zero_counter) = mod(k,self.fs/self.fm)+self.fs/self.fm/2;
                            if zero_counter == 100
                                break;
                            end
                            zero_counter = zero_counter + 1;

                        end
                        k = k+1;
                    end
                   
                end
                %figure(1);
                %plot(mulLP);
                %xlim([0 2000]);
                %offset = round(mean(offset_vector(1:zero_counter)));
                offset = mode(offset_vector(1:zero_counter));
                %disp(offset_vector(1:zero_counter))
                % Demodulate signal
                s_demod = zeros(1,floor(self.l_buffer/(self.fs/self.fm)));
                j = 1;
                for i=offset:self.fs/self.fm:self.l_buffer
                    try
                        s_demod(j) = (sign(mulLP(round(i))) + 1)/2;
                    catch
                        disp('Demodulation offset error...')
                        disp('j:')
                        disp(j)
                        disp('Lenght(s_demod:')
                        disp(length(s_demod))
                        disp('Offset vector:')
                        disp(offset_vector')
                        disp('Lenght of mulLP:')
                        disp(length(mulLP))
                        disp('i:')
                        disp(i)
                    end
                    j = j+1;
                end
                
                if self.plot_detectionPoints
                    % Generate offset vector to plot detection points
                    offsetVector = ones(1,length(mulLP)) - 5;
                    for l = offset:self.fs/self.fm:length(mulLP)
                        offsetVector(l) = 0.6;
                    end
                    
                    figure(1);
                    plot(mulLP);
                    hold on;
                    plot(offsetVector,'*', 'Color', 'red');
                    %plot(c);
                    xlim([49000 50000]);
                    ylim([-1 1]);
                    
                    %hold off;
                    title('Multiplied signal with desicion points');
                end
                
                
                %offset = mean(offset_vector(1:zero_counter));
            end
            
        end
        
        function plotDetectionPoints(self,enable)
            if enable > 0
                self.plot_detectionPoints = true;
            else
                self.plot_detectionPoints = false;
            end
        end
        
        function setHd(self,Hd)
            self.Hd = Hd;
        end
    end
    
end

