classdef Modulator < handle
    %BPSKMODULATOR Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        fs;
        fc;
        fm;
        n_ch;
        code;
        
        p;
        time;
        
        n_code;
        l_code;
        l_signal;
        
        c;
        
        zero_buffer;
        buffer;
        
        code_idx = 0;
        offset = 0;
    end
    
    methods
        function self = Modulator(fs, fc, fm, code)
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.code = code;
            
            self.p = self.fs/self.fm;
            [self.n_code, self.l_code] = size(code);
            self.l_signal = self.p*self.l_code;
            
            % Create carrier and modulate
            T = (self.l_signal-1)/self.fs;
            t = 0:1/fs:T;
            self.c = cos(2*pi*self.fc.*t);
            
            self.zero_buffer = zeros(1,2*self.l_signal);
            self.buffer = self.zero_buffer;
            
        end
        
        function signal = step(self)
            self.buffer(1:self.l_signal) = self.buffer(self.l_signal + 1: 2*self.l_signal);
            self.buffer(self.l_signal + 1 : 2*self.l_signal) = self.zero_buffer(1:self.l_signal);
            if self.code_idx > 0
               new_signal = self.zero_buffer;
               new_signal(self.offset:self.l_signal + self.offset-1) = self.generateCode(self.code_idx);
               self.buffer = self.buffer + new_signal;
               fprintf(': Output Code %i \n', self.code_idx);
               self.code_idx = 0;
              
            end
                
            
            signal = self.buffer(1:self.l_signal);
            
            
        end
        
        function setOutput(self, code_idx, offset)
            
            self.code_idx = code_idx;
            self.offset = offset;
            
        end
        
        function [s,m] = generateCode(self, code_idx)
            
            [s,m] = self.modulate(self.code(code_idx,:));
            
        end
        
        function [s,m] = modulate(self,m)
            % Transform to BPSK
            m_mod = 2.*m - 1;
            %m_mod=m;
            % Upsampling
            phi = zeros(1,self.l_signal);
            for i = 1:self.l_code
                phi(self.p*i-(self.p-1)) = m_mod(i);
                for j = 1:self.p-1
                    phi(self.p*i-(self.p-1)+j) = m_mod(i);
                end
            end
                  
            m = phi;
            s = phi.*self.c;
           
        end
    end
    
end

