classdef Node < handle
    %BUFFERPROCESS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        % Single initialisation
        nodeId;
        n_node;
        fs;
        fs_oversample;
        fc;
        fm;
        l_signal;
        n_ch;
        baseband;
        code;
        
        % Filter stuff
        N = 100;
        fp = 2*1000;
        rp = 0.001;
        rst = 1E-6;
        fir;
       
        n_code;
        l_buffer;
        l_code;
        
        % Input buffer
        buffer;
        
        % Positions
        mobdev;
        positions;
        
        % Objects
        modulator;
        demodulator;
        correlator;
        timekeeper;
        communicator;
        positioner;
        
        baseband_oversample = 6;
        
        handles;
        
        
    end
    
    methods
        function self = Node(nodeId, n_node,l_code, fs, fc, fm, l_signal, baseband, handles)
            % Assign input variables
            self.nodeId = nodeId;
            self.n_node = n_node;
            self.l_code = l_code;
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.l_signal = l_signal;
            self.n_ch = 2;
            self.baseband = baseband;
            self.handles = handles;
            
            
            % Initialize
            if baseband
                self.fs_oversample = self.baseband_oversample*(fc+fm);
            end
            
            self.code = getGold(1,3*self.n_node, self.l_code);
            
            % Generate Low-pass filter
            self.fir = firDesign(self.N, self.fp, self.fs, self.rp, self.rst);
            
            [self.n_code, self.l_code] = size(self.code);
            self.l_buffer = 2*self.l_signal;%ceil(fs/fc*self.l_code);
            
            
            self.buffer = zeros(self.n_ch, self.l_buffer);
            
            self.modulator = Modulator(self.fs, self.fc, self.fm, self.code);
            %self.demodulator = Demodulator(self.fs, self.fc, self.fm, self.l_signal, self.l_buffer, self.baseband, self.fs_oversample, self.fir);
            self.demodulator = Demodulator_complex(self.fs, self.fc, self.fm, self.l_signal, self.l_buffer);
            %self.correlator = Correlator(self.fs, self.fc, self.fm, self.l_signal, self.baseband, self.fs_oversample, self.code);
            self.correlator = Correlator_complex(self.fs, self.fc, self.fm, self.l_buffer, self.code);
            self.timekeeper = TimeKeeper(self.n_code, self.n_ch,self.l_signal, self.fs, self.l_signal);
            self.communicator = Communicator(self.nodeId, self.fs, self.fc, self.fm, self.l_signal, self.code, self.n_code, self.modulator);
            self.positioner = Positioner(self.n_node, self.handles);
            
        end
        
        function out = step(self,in)
            % Load buffer with data from previous buffer
            self.buffer(:,1:self.l_signal) = self.buffer(:,end-self.l_signal+1:end);
            self.buffer(:,self.l_signal+1:end) = in;
            
            detection = zeros(self.n_code, 3*self.n_ch);
            offset = zeros(1,self.n_ch);
            
            for current_ch = 1:self.n_ch
                % Demodulate
                [s_demod, r_lp] = self.demodulator.step(self.buffer(current_ch,:));
                detection(:,(3*current_ch - 2):3*current_ch) = self.correlator.step(s_demod,r_lp);
                %det{current_ch} = self.correlator.step(s_demod);
            end
            %detection = [det{1} det{2}];
            %disp(detection)
            [updates, current_lag] = self.timekeeper.step(detection,offset);
            USpositions = self.communicator.step(updates, current_lag);
            self.positions = self.positioner.step(USpositions);
            out = self.modulator.step;
            
            
            
        end
        
        function [n_codes, n_channels] = getDetectionInfo(self)
           n_codes = self.n_code;
           n_channels = self.n_ch;
        end
        
        function printDetected(self)
           self.timekeeper.printDetected(); 
        end
        
        function sendRequest(self)
            self.communicator.sendRequest();
        end
        
        function calibrate(self)
            self.communicator.calibrate();
        end
    end
    
end

