classdef TimeKeeper < handle
    %TIMEKEEPER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        lags;
        n_lags;
        
        step_time;
        current_time;
        
        previous_code_time;
        previous_code_thres;
        
        current_buffer;
        n_code;
        n_ch;
        l_frame;
        fs;
        l_signal;
        
    end
    
    methods
        function self = TimeKeeper(n_code, n_ch, l_frame, fs, l_signal)
                        
            self.n_code = n_code;
            self.n_ch = n_ch;
            self.l_frame = l_frame;
            
            self.fs = fs;
            self.l_signal = l_signal;
            
            self.step_time = self.l_signal/self.fs;
            
            self.current_buffer = 1;
            self.n_lags = 0;
            
            self.current_time = 0;
            
            self.previous_code_time = zeros(n_code, self.n_ch) - 2*self.l_signal;
            self.previous_code_thres = zeros(n_code, self.n_ch);
         
        end
        
        function [updates, current_lag] = step(self,detection,offset)
            % Takes detectin as Detected, lag, threshold
            % Returns updates as
            % Channel, Code, Total lag
            % and current_time as total lag.
            self.current_time = self.current_time + self.l_signal;
            start_idx = self.n_lags;
            for i = 1:self.n_code
                for j = 1:self.n_ch
                    if detection(i,3*j-2) > -1  % A signals has been detected for the node
                        self.n_lags = self.n_lags + 1;
                        self.lags(self.n_lags,1) = j; %Ch
                        self.lags(self.n_lags,2) = i; %Code
                        self.lags(self.n_lags,3) = (self.current_buffer - 1)*self.l_frame + detection(i,3*j-1) + offset(j); % Total lag
                        self.lags(self.n_lags,4) = detection(i,3*j-1) + offset(j); % Buffer lag
                        self.lags(self.n_lags,5) = detection(i,3*j); % Threshold
                    end
                end
            end
            end_idx = self.n_lags;
            updates = [];
            if end_idx > start_idx
                detect = self.lags(start_idx + 1 : end_idx,:);
                counter = 1;
                %disp(detect)
                
                for i = 1:size(detect,1);
                    if ( self.previous_code_time(detect(i,2),detect(i,1)) < detect(i,3)-self.l_signal ) || ( self.previous_code_thres(detect(i,2),detect(i,1)) < detect(i,5) )
                        self.previous_code_time(detect(i,2),detect(i,1)) = detect(i,3);
                        self.previous_code_thres(detect(i,2),detect(i,1)) = detect(i,5);
                        updates(counter,:) = [detect(i,1), detect(i,2), detect(i,3)];
                        fprintf('Code %3i updated to %i on channel %i at threshold %3.0i.\n',detect(i,2),detect(i,3),detect(i,1), round(detect(i,5)));
                        counter = counter + 1;
                    end
                end
            end
            self.current_buffer = self.current_buffer + 1;
            current_lag = self.current_time;
        end
        
        
        
        
        function printDetected(self)
            if self.n_lags > 0
                fprintf('Detection printout from timekeeper:\n');
                for i = 1:self.n_lags
                    fprintf('Ch %i: Code %3i at lag %i\n',self.lags(i,1), self.lags(i,2), self.lags(i,3));
                    
                end
            else
                fprintf('No lags..\n');
            end
            
            
        end
        
        
    end
    
end

