function [ m ] = demodBPSK( s, L, fs, fc, fm, FIR)
%demodBPSK Summary of this function goes here
%   Detailed explanation goes here

% Find phase, multiply by local carrier
ratio1 = fs/fc;
ratio2 = fs/fm;

LmaxCorr = 1000*ratio1;   % Nsamples for carrier detect

t = 0:1/fs:(L+LmaxCorr)/fs; % Length of m + crosscorr
c = sin(2*pi*fc*t);         % Carrier

% Crosscorrelate local carrier and signal
xc = xcorr(c(1:LmaxCorr),s(1:LmaxCorr));
[xcMax, xcMaxIndex] = max(xc); %Find corr. peak

% Use peak index to offset carrier. Remove extra.
% Some uncertainty around zero offset. Needs attention.
offset = length(xc)/2-xcMaxIndex;
if offset > 0
    offset = ceil(offset);
    c = c(LmaxCorr-offset+1:end-offset-1);
    
else
   offset = floor(offset); 
   c = c(1-offset:end-offset-1-LmaxCorr);
end
    
if 0
    % Plot sample of signal and offset carrier
    figure
    plot(s(1:LmaxCorr))
    hold on;
    plot(c(1:LmaxCorr),'*', 'Color', 'Red')
    title('Sample signal');
end
if 0
    % Plot carrier correlation
    figure
    plot(xc);
    title('Phase correlation')
end


% Multiply signal by offset carrier.
mul = s.*c;

% Low-pass signal
mulLP = step(FIR,mul');

% Find decision points by zero-detection
% find max -> find zero
[mulLPmax mulLPmaxIdx] = max(mulLP);
k = mulLPmaxIdx;
while k<L-1
    if (sign(mulLP(k)) * sign(mulLP(k+1))) == -1;
        break;
    end
    k = k+1;
end

% Find offset by modulus with known ratio fs/fm
offset = mod(k,ratio2)+ratio2/2;
%offset = 5; Bit-accurate, for reference test.

% Generate offset vector to plot detection points
offsetVector = ones(1,length(mulLP)) - 5;
for l = offset:ratio2:length(mulLP)
    offsetVector(l) = 0.6;
end

if 0
    % Plot detection points
    figure(3);
    plot(mulLP);
    hold on;
    plot(offsetVector,'*', 'Color', 'red');
    %xlim([0 10000]);
    ylim([-1 1]);
    hold off;
    title('Multiplied signal with desicion points');
end


% Demodulate signal
m = zeros(1,floor(L/ratio2));
j = 1;
for i=offset:ratio2:L
    m(j) = (sign(mulLP(i)) + 1)/2;
    j = j+1;
end


end
