classdef Detector
    %DETECTOR Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
    fs;
    fc;
    fm;
    l_buffer;
    preamble;
    code;
    
    fir;
    N_fir;
    
    c;
    
    D;
    Nc_pr_m;
    Ns_pr_c;
    
    N_code;
    L_code;
    L_xc;
    L_dec;
    
    min_thr = 5; %7
    
    
    end
    
    methods
        function self = Detector(fs, fc, fm, preamble, code)
            
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.preamble = preamble;
            self.code = code;
            
            % Initialize
                % FIR LP
                self.N_fir = 100;
                fp = self.fm/6;
                rp = 0.001;
                rst = 1E-6;
                self.fir = firDesign(self.N_fir, fp, fs, rp, rst);
                
                % Code
                [self.N_code, self.L_code] = size(code);
                
                % Carrier
                self.D = self.fs/self.fm;
                self.Nc_pr_m = self.fc/self.fm;
                self.Ns_pr_c = self.fs/self.fc;
                
                t = (0:self.L_code*self.D*2*2-1)/self.fs;
                self.c = exp(-1i*2*pi*self.fc.*t);
                
                
                self.L_xc = self.L_code*8-1;
                self.L_dec = self.L_code*4;
                
                
            
        end
        
        function detection = step(self, r)
            % jaja
            detection = 0;
            
            % DOWN_MIX input signal
            r_dm = self.c.*r;
            r_dmlp = self.fir.step(r_dm')';
            
           
            % CORRELATE the preamble code
            r_dec = zeros(self.Nc_pr_m,self.L_dec);
            pre_xc_abs = zeros(self.Nc_pr_m,self.L_xc);
            pre_max = zeros(self.Nc_pr_m, 1);
            pre_max_idx = zeros(self.Nc_pr_m, 1);
            
            for i = 1:self.Nc_pr_m;
            % For each decimation selection
                r_dec(i,:) = r_dmlp((i-1)*self.Ns_pr_c+1:self.D:end);
                pre_xc_abs(i,:) = abs(xcorr(r_dec(i,:),self.preamble));
                [pre_max(i), pre_max_idx(i)] = max(pre_xc_abs(i,:));
            end

            [ ~, pre_sel_idx] = max(pre_max);
            
            
            % FIND message offset
            pre_lag_diff = pre_max_idx - pre_max_idx(pre_sel_idx); % diff between selected and others
            iv = find(pre_lag_diff ~= 0);
            
            if ~isempty(iv)
                if iv(1) > pre_sel_idx
                    start = iv(1) - 1;
                    center = start-2.5;
                    offset = pre_sel_idx - center;
                else
                    start = iv(end) + 1;
                    center = start + 2.5;
                    offset = pre_sel_idx - center;
                end
            else
                offset = pre_sel_idx-3.5;
            end
            
            if 0
                pre_max_idx
                iv         
                pre_lag_diff
                offset
            end
            
            
                
            
            
            % CORRELATE the unique code
            code_xc = zeros(self.N_code,self.L_xc);
            code_xc_abs = zeros(self.N_code,self.L_xc);
            code_max = zeros(self.N_code, 1);
            code_max_idx = zeros(self.N_code, 1);
            code_mean = zeros(self.N_code,1);
            code_detect = zeros(self.N_code,1) -1;
            code_lag_coarse = zeros(self.N_code,1);
            code_lag_fine = zeros(self.N_code,1);
            code_lag = zeros(self.N_code,1);
            code_thr = zeros(self.N_code,1);
            ang_xc = zeros(self.N_code,1);
            ang_xc_plusone = zeros(self.N_code,1);
            
            for i = 1:self.N_code
            % For each code, calculate xc and find max
                code_xc(i,:) = xcorr(self.code(i,:),r_dec(pre_sel_idx,:));
                code_xc_abs(i,:) = abs(code_xc(i,:));
                [code_max(i), code_max_idx(i)] = max(code_xc_abs(i,:));
                code_mean(i) = mean(code_xc_abs(1,:));
                code_thr(i) = code_max(i)/code_mean(i);
                if code_thr(i) > self.min_thr
                    code_detect(i) = 1;
                    % Calculate coarse phase offset
                    code_lag_coarse(i) = (self.L_xc/2 - code_max_idx(i)) * self.D - self.N_fir/2 - self.L_code*self.D + (pre_sel_idx - offset)*self.Ns_pr_c - 2.5;
                    % Calculate fine phase offset using phase unwrap
                    ang_xc(i) = angle(code_xc(i,code_max_idx(i)));
                    ang_xc_plusone(i) = angle(code_xc(i,code_max_idx(i)+1));
                    if ang_xc(i) > ang_xc_plusone(i) && ang_xc(i) > 0.01
                        amb = 2*pi;
                    else
                        amb = 0;
                    end
                    code_lag_fine(i) = (ang_xc(i) - amb)/(2*pi)*self.Ns_pr_c;
                    code_lag(i) = round(code_lag_coarse(i) + code_lag_fine(i));
                    
                end
            end
           
             
            
            
            detection = [code_detect,code_lag, code_thr];
            
            
             
            if 1
                % Prints
                fprintf('Pre_sel_idx = %i\n',pre_sel_idx);
                fprintf('Code_thr = %f\n', code_thr(1));
                fprintf('code_lag_coarse = %i\n', code_lag_coarse(1));
                fprintf('fine_lag = %f\n', code_lag_fine(1));
                fprintf('code_lag = %.1f\n\n', code_lag_coarse(1)+code_lag_fine(1));
            end
            
            if 0
                % Plot code correlation
                figure;
                plot(code_xc_abs(1,:));
            end
            
            if 0
                % Plot angle
                figure;
                plot([-2 -1 0 1 2],angle(code_xc(1,code_max_idx(1) - 2:code_max_idx(1) + 2)))
                grid on;
            end
            
            
            
            
        end
            
    end
    
end

