classdef Detector2
    %DETECTOR Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
    fs;
    fc;
    fm;
    l_buffer;
    preamble;
    code;
    
    fir;
    N_fir;
    
    c;
    
    D;
    Nc_pr_m;
    Ns_pr_c;
    
    N_code;
    L_code;
    L_xc;
    L_dec;
    
    min_thr = 22;
    
    preamble_fs;
    L_xc_fs;
    std_offset;
    pre_lag_max;
    end
    
    methods
        function self = Detector2(fs, fc, fm, preamble, code)
            
            self.fs = fs;
            self.fc = fc;
            self.fm = fm;
            self.preamble = preamble;
            self.code = code;
            
            % Initialize
                % FIR LP
                self.N_fir = 100;
                fp = self.fm/6;
                rp = 0.001;
                rst = 1E-6;
                self.fir = firDesign(self.N_fir, fp, fs, rp, rst);
                
                % Code
                [self.N_code, self.L_code] = size(code);
                
                % Carrier
                self.D = self.fs/self.fm;
                self.Nc_pr_m = self.fc/self.fm;
                self.Ns_pr_c = self.fs/self.fc;
                
                t = (0:self.L_code*self.D*2*2-1)/self.fs;
                self.c = exp(-1i*2*pi*self.fc.*t);
                
                
                self.L_xc = self.L_code*8-1;
                self.L_dec = self.L_code*4;
                
                
            % Full-sample-rate preamble correlation
                % Upsample the preamble code
                self.preamble_fs = upsample(self.preamble, self.D);
                for i = 1:self.D:length(self.preamble_fs)
                    self.preamble_fs(i+1:i+self.D-1) = self.preamble_fs(i);
                end
                
                % Length of full-sample-rate correlation
                self.L_xc_fs = 8*self.L_code*self.D-1;
                
                % Standard offset calculated by fir filter length and
                %   code sample offset
                self.std_offset = mod(self.N_fir+self.D/2,self.D);
                
                % Preamble maximum lag (unique code out of buffer)
                self.pre_lag_max = self.L_code * 2 * self.D;
        end
        
        function detection = step(self, r)

            % DOWN_MIX input signal
            r_dm = self.c.*r;
            r_dmlp = self.fir.step(r_dm')';
            
            if 0 
                % RMS readout to command window:
                p = rms(r_dmlp);
                L_p = 10*log10(p);
                fprintf('RMS: %4.2f dB\n',L_p);
            end
            
            % CORRELATE the preamble at full-sample
            pre_fs_xc = xcorr(self.preamble_fs,r_dmlp);
            pre_fs_xc_abs = abs(pre_fs_xc);
            [pre_fs_max, pre_fs_max_idx] = max(pre_fs_xc_abs);
            pre_lag = round((self.L_xc_fs/2 - pre_fs_max_idx) - self.N_fir/2 + 0.5);
            pre_fs_xc_mean = mean(pre_fs_xc_abs);
            pre_fs_xc_thr = pre_fs_xc_abs(pre_fs_max_idx)/pre_fs_xc_mean;
            
            
            % COMPUTE decimation offset
            dec_offset = mod(pre_lag+self.std_offset, self.D) + 1;
  
            % DECIMATE
            r_dec = r_dmlp(dec_offset:self.D:end);
 
            % CORRELATE the unique code
            code_xc = zeros(self.N_code,self.L_xc);
            code_xc_abs = zeros(self.N_code,self.L_xc);
            code_max = zeros(self.N_code, 1);
            code_max_idx = zeros(self.N_code, 1);
            code_mean = zeros(self.N_code,1);
            code_detect = zeros(self.N_code,1) -1;
            code_thr = zeros(self.N_code,1);
            code_lag = zeros(self.N_code,1);
            detection = zeros(self.N_code,3)-1;
            
            
            
            if pre_lag < self.pre_lag_max && pre_lag > -1 && pre_fs_xc_thr > self.min_thr
                for i = 1:self.N_code
                % For each code, calculate xc and find max
                    code_xc(i,:) = xcorr(self.code(i,:),r_dec);
                    code_xc_abs(i,:) = abs(code_xc(i,:));
                    [code_max(i), code_max_idx(i)] = max(code_xc_abs(i,:));
                    code_mean(i) = mean(code_xc_abs(1,:));
                    code_thr(i) = code_max(i)/code_mean(i);
                    
                end
                
                [max_corr, max_corr_idx] = max(code_thr);
                %if max_corr > 20
                    detection(max_corr_idx,:) = [1, pre_lag, code_thr(max_corr_idx)];
                %end
            end
           
             if 1
                figure;
                plot(code_xc_abs);
            end
            
             
            if 0
                % Prints
                fprintf('Code_thr = %f\n', code_thr(1));
                fprintf('lag = %f\n', code_lag(1));
                fprintf('pre_thr = %.2f\n',pre_fs_xc_thr);
            end
            
            if 0
                fprintf('Pre lag: %i\n',pre_lag);
                fprintf('Pre lag thr: %i\n',pre_fs_xc_thr);
                fprintf('Pre_fs_max_idx: %i\n',pre_fs_max_idx);
            end
            
            if 1
                % Plot code correlation
                figure;
                plot(code_xc_abs(1,:));
                title('Code correlation');
            end
            
            if 0
                figure;
                plot(pre_fs_xc_abs);
                title('Preamble Correlation');
            end

            
            
            
        end
            
    end
    
end

