classdef Node < handle
    %BUFFERPROCESS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        % Single initialisation
        nodeId;
        N_node;
        fs;
        fc;
        fm;
        L_signal;
        
        preamble;
        code;
       
        N_code;
        L_buffer;
        L_code;
        D;
        
        % Input buffer
        buffer;
        
        % Positions
        mobdev;
        positions;
        
        % Objects
        modulator;
        detector_1;
        detector_2;
        timekeeper;
        communicator;
        positioner;
        
        handles;
        
        level_calibration = 0;
        
    end
    
    methods
        function self = Node(nodeId, N_node, fs, fc, fm, handles)
            % Assign input variables
                self.nodeId = nodeId;
                self.N_node = N_node;
                self.fs = fs;
                self.fc = fc;
                self.fm = fm;
                self.handles = handles;
            
            % Set options
                self.L_code = 511;
            
            % Initialize
            
                % The number codes is three times the number of nodes
                self.N_code = 3*self.N_node;
            
                % The first code is the preamble
                self.preamble = getGold(510,1,self.L_code); 
            
                % The second code and up is the unique set.
                self.code = getGold(1,self.N_code, self.L_code);
                
                % Init the input buffer using the up-/down-sample factor D
                self.D = self.fs/self.fm;
                % The signal length is the up-sample of two code lengths
                self.L_signal = self.D * self.L_code * 2;
                % The buffer is twice the length of one complete signal
                self.L_buffer = self.L_signal * 2;   
                self.buffer = zeros(2, self.L_buffer); % Two channels
                
            % Initialize the processing objects
                self.modulator = Modulator(self.fs, self.fc, self.fm, self.code);
                self.detector_1 = Detector2(self.fs, self.fc, self.fm, self.preamble, self.code);
                self.detector_2 = Detector2(self.fs, self.fc, self.fm, self.preamble, self.code);
                self.timekeeper = TimeKeeper(self.N_code, self.fs, self.L_signal);
                self.communicator = Communicator(self.nodeId, self.fs, self.fc, self.fm, self.L_signal, self.code, self.N_code, self.modulator);
                self.positioner = Positioner(self.N_node, self.handles);
            
        end
        
        function out = step(self,in)
            % Load buffer with data from previous buffer
                self.buffer(:,1:self.L_signal) = self.buffer(:,end-self.L_signal+1:end);
                self.buffer(:,self.L_signal+1:end) = in;
            
            % Init the detection variable
                if self.level_calibration
                    out = self.modulator.continousOutput;
                    detection_1 = self.detector_1.step(self.buffer(1,:));
                    detection_2 = self.detector_2.step(self.buffer(2,:));
                else
                    detection_1 = self.detector_1.step(self.buffer(1,:));
                    detection_2 = self.detector_2.step(self.buffer(2,:));

                    detection = [detection_1 detection_2];

                    [updates, current_lag] = self.timekeeper.step(detection);
                    USpositions = self.communicator.step(updates,current_lag);
                    self.positions = self.positioner.step(USpositions);
                    out = self.modulator.step;
                end
                
                
                
            
        end
        
        
        function printDetected(self)
           self.timekeeper.printDetected(); 
        end
        
        function sendRequest(self)
            self.communicator.sendRequest();
        end
        
        function calibrate(self)
            self.communicator.calibrate();
        end
    end
    
end

