classdef Positioner < handle
    %POSITIONER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        mobdev;
        aziCal;
        aziEnable;
        
        handles;
        
        n_node;
        
        us_positions;
        positions;
        
        angle = 0;
        
        skip_step;
        
        % Figure properties
        max_range = 5; % Maximum range of plot
        figure_h;
        axes_h;
        node_pos_h;
        direction_vec_h;
        
        % Logging
        log;
    end
    
    methods
        function self = Positioner(n_node, handles)
            self.n_node = n_node;
            self.handles = handles;
            
            self.us_positions = zeros(self.n_node, 2);
            
            if isa(self.handles.mobdev,'mobiledev')
                self.mobdev = handles.mobdev;
                self.mobdev.OrientationSensorEnabled = 1;
                self.mobdev.AccelerationSensorEnabled = 0;
                self.mobdev.AngularVelocitySensorEnabled = 0;
                self.mobdev.MagneticSensorEnabled = 0;
                self.mobdev.PositionSensorEnabled = 0;
            
                self.mobdev.Logging = 1;
                self.aziCal = self.mobdev.Orientation(1);
                self.aziEnable = 1;
            else
                self.aziEnable = 0;
            end
            
            self.us_positions = ones(self.n_node,2)*1;
            
            self.createFigure();
        end
        
        function positions = step(self, USpositions)
            % Input USpositions as NodeId, Distance, Angle
            if self.aziEnable
                self.angle = mod(self.mobdev.Orientation(1)+180 - self.aziCal,360)-180;
                %self.handles.current_angle.String = sprintf('%3.2f', self.angle);
                self.updateAngle(); 
            end
            
            
            for i = 1:size(USpositions,1)
               
               node_id = USpositions(i,1);
               dist = USpositions(i,2);
               ang = USpositions(i,3);
               
               abs_ang = USpositions(i,3) + self.angle;
               
               fprintf('Node %2i at %2.2f meter, angle %3.2f and abs angle %3.2f\n', node_id, dist, ang, abs_ang);
               self.log(end+1,:) = [node_id,dist,ang,abs_ang];
               
               if 1%abs( abs_ang - self.us_positions(node_id,2) ) < 180
                   
                   self.us_positions(node_id,1) = dist;
                   self.us_positions(node_id,2) = abs_ang;
               end
               
               self.updateNodePosition(node_id);
               
            end
            
            
            positions = []; 
        end
        
        function createFigure(self)
           self.figure_h = figure(2);
           %self.figure_h.Position = [0 1050 900 800];
           self.axes_h = axes;
           self.axes_h.XLim = [-self.max_range self.max_range];
           self.axes_h.YLim = [-self.max_range self.max_range];
           title('Static room representation');
           xlabel('x-distance [m]');
           ylabel('y-distance [m]');
           hold all;
           
           % Plot circle
           r = 0.22;
           ang=0:0.01:2*pi;
           xp=r*cos(ang);
           yp=r*sin(ang);
           plot(xp,yp,'LineWidth', 2);
           grid on;
           % Plot direction vector
           self.direction_vec_h = plot([0, 0], [0, 0.5], 'Color', 'blue', 'LineWidth', 2);
           
           
           for i = 1:self.n_node
              self.node_pos_h{i} = plot(self.max_range+2,self.max_range+2, 'o');
           end
           
           
        end
        
        function updateAngle(self)
            x = self.max_range/10*sind(self.angle);
            y = self.max_range/10*cosd(self.angle);
            self.direction_vec_h.XData(2) = x;
            self.direction_vec_h.YData(2) = y;
            
        end
        
        function updateNodePosition(self, node_id)
            dist = self.us_positions(node_id,1);
            ang = self.us_positions(node_id,2);
            if dist > 0
                x = dist*sind(ang);
                y = dist*cosd(ang);
                self.node_pos_h{node_id}.XData = real(x);
                self.node_pos_h{node_id}.YData = real(y);
            end
        end
        
        function log = getLog(self)
            log = self.log;
        end
    end
    
end

